/*
 * Copyright (c) 2018-2020 REV Robotics
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of REV Robotics nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <stdint.h>

extern "C" {

/**
 * Add a device to be updated
 * 
 * @param numDevicesToAdd The total number of devices you wish to update. This must match the number of motor controller IDs
 * you pass into the next parameter
 * 
 * @param ... The device IDs you wish to update as a list of any size. Length must match the numDevicesToAdd parameter
 */
void c_SparkMax_AddDeviceSWDL(int numDevicesToAdd, ...);

/**
 * Begin the software update for the sparks. You can call this function repeatedly to get the status of the update
 * 
 * @param fileName The filename of the bin file to send to the Spark MAXs
 * 
 * @return The percent complete of the download
 */
int32_t c_SparkMax_RunSWDL(const char* fileName);


/**
 * Reset the software download state to prepare another run. Must
 * be called after complete download.
 * 
 * @return Whether or not SWDL was reset
 */
bool c_SparkMax_ResetSWDL();

}  // extern "C"
